import os
import sys
import glob
import click
import random
import logging
import pandas as pd
import uuid


def save_merged_df(df, path, file, logger):
    try:
        df.to_excel(
            f"{path}{os.path.sep}anonymisoitu_{file['filename'][:-5]}.xlsx",
            index=False,
        )
        logger.info(
            f"Writing merged data file {path}{os.path.sep}raportti_{file['filename'][:-5]}.xlsx succesfully."
        )
    except Exception as error:
        logger.critical(
            f"Saving merged data to file {path}{os.path.sep}raportti_{file['filename'][:-5]}.xlsx failed. error: {error}"
        )
        pass


def create_uuids(count):
    list_of_uuids = list()
    for _ in range(count):
        list_of_uuids.append(uuid.uuid4())
    return list_of_uuids


def create_key_dict(ids, key_dict, surrogates):
    for id in ids:
        if id not in key_dict:
            index = round(random.random() * len(surrogates))
            key_dict[id] = surrogates[index]
            surrogates.remove(surrogates[index])


@click.command()
@click.argument("output_path", type=click.Path(exists=True))
def main(output_path):

    logger = logging.getLogger(__name__)
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    )

    files = []

    try:
        for file in glob.glob(f"{output_path}{os.path.sep}*.xlsx"):
            files.append(
                {"relative_pathname": file, "filename": file.split(os.path.sep)[-1]}
            )
    except Exception as error:
        logger.critical(
            f"Reading output filenames from relative path {output_path} failed. error: {error}"
        )
        sys.exit(0)

    key_dict = dict()
    surrogates = create_uuids(50000)
    for file in files:
        df = None
        try:
            df = pd.read_excel(file["relative_pathname"])
            logger.info(
                f"Reading KOSKI data file {file['relative_pathname']} to the dataframe succesfully."
            )
        except Exception as error:
            logger.critical(
                f"Reading koski data from file {file['relative_pathname']} failed. error: {error}"
            )
            sys.exit(0)
        student_ids = list(set(df["Opiskeluoikeuden oid"]))
        create_key_dict(student_ids, key_dict, surrogates)
        df_surrogates = pd.DataFrame.from_dict(
            key_dict, orient="index", columns=["opiskelija"]
        )
        merged = df.merge(
            df_surrogates, how="left", left_on="Opiskeluoikeuden oid", right_index=True
        )
        merged = merged.drop(
            labels=[
                "Opiskeluoikeuden oid",
                "Lähdejärjestelmä",
                "Opiskeluoikeuden tunniste lähdejärjestelmässä",
                "Sisältyvät opiskeluoikeudet",
                "Sisältyy opiskeluoikeuteen",
                "Oppijan oid",
                "Sukunimi",
                "Etunimet",
            ],
            axis=1,
        )
        save_merged_df(merged, output_path, file, logger)


if __name__ == "__main__":
    main()  # pylint: disable=no-value-for-parameter
